@@ -4,6 +4,7 @@
 
 #include <config.h>
 #include <panel-applet.h>
+#include <egg-screen-help.h>
 #include "charpick.h"
 
 
@@ -350,7 +351,9 @@ static void applet_change_orient(PanelApplet *applet, PanelAppletOrient o, gpoin
 
 
 static void
-about (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
+about (BonoboUIComponent *uic,
+       charpick_data     *curr_data,
+       const char        *verb)
 {
   static GtkWidget *about_box = NULL;
   GdkPixbuf   	   *pixbuf;
@@ -370,11 +373,11 @@ about (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 
   const gchar *translator_credits = _("translator_credits");
 
-  if (about_box != NULL)
-  {
-	  gdk_window_show(about_box->window);
-	  gdk_window_raise(about_box->window);
-	  return;
+  if (about_box) {
+	gtk_window_set_screen (GTK_WINDOW (about_box),
+			       gtk_widget_get_screen (curr_data->applet));
+	gtk_window_present (GTK_WINDOW (about_box));
+	return;
   }
   
   file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "charpick.png", FALSE, NULL);
@@ -400,6 +403,8 @@ about (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
   if (pixbuf) 
   	gdk_pixbuf_unref (pixbuf);
    
+  gtk_window_set_screen (GTK_WINDOW (about_box),
+			 gtk_widget_get_screen (curr_data->applet));
   gtk_window_set_wmclass (GTK_WINDOW (about_box), "character palette", "Character Palette");
   gnome_window_icon_set_from_file (GTK_WINDOW (about_box), GNOME_ICONDIR"/charpick.png");
 
@@ -411,12 +416,17 @@ about (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 
 
 static void
-help_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
+help_cb (BonoboUIComponent *uic,
+	 charpick_data     *curr_data,
+	 const char        *verb)
 {
   GError *error = NULL;
-  gnome_help_display("char-palette",NULL,&error);
 
-  if (error) {
+  egg_screen_help_display (
+		gtk_widget_get_screen (curr_data->applet),
+		"char-palette", NULL, &error);
+
+  if (error) { /* FIXME: the user needs to see this */
     g_warning ("help error: %s\n", error->message);
     g_error_free (error);
     error = NULL;
@@ -447,9 +457,9 @@ applet_destroy (GtkWidget *widget, gpointer data)
 }
 
 static const BonoboUIVerb charpick_applet_menu_verbs [] = {
-        BONOBO_UI_VERB ("Preferences", property_show),
-        BONOBO_UI_VERB ("Help", help_cb),
-        BONOBO_UI_VERB ("About", about),
+        BONOBO_UI_UNSAFE_VERB ("Preferences", show_preferences_dialog),
+        BONOBO_UI_UNSAFE_VERB ("Help",        help_cb),
+        BONOBO_UI_UNSAFE_VERB ("About",       about),
 
         BONOBO_UI_VERB_END
 };
